/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
/*
 *  $RCSfile: $
 *  $Revision: $  $Date: $ 
 */
/**
 * 
 */
package com.ibm.ram.internal.batch.filesystem;

import java.io.File;
import java.util.*;

import org.apache.tools.ant.DirectoryScanner;
import org.eclipse.core.runtime.IProgressMonitor;

import com.ibm.ram.common.data.Asset;
import com.ibm.ram.internal.batch.filesystem.ui.RuleUtil;
import com.ibm.ram.internal.rich.ui.util.Messages;

/**
 * An abstract base class for rules.
 * 
 * @author ebordeau
 * @since 7.1
 */
public abstract class AbstractRule implements IRule {
	
	private String filter;
	private String value;
	private IRuleContainer parent;
	
	private IRuleDelegate getDelegate(Object object) {
		IRuleDelegate[] delegates = RuleDelegateExtensionManager.getDelegates();
		IRuleDelegate delegate = null;
		for (int i = 0; i < delegates.length; i++) {
			if (delegates[i].canHandle(object, this)) {
				if (delegate == null || delegates[i].getPriority() > delegate.getPriority())
					delegate = delegates[i];
			}
		}
		return delegate;
	}
	
	public List apply(Object object, IProgressMonitor monitor) {
		IRuleDelegate delegate = getDelegate(object);
		if (delegate != null) {
			FileSystemDataSource dataSource = (FileSystemDataSource)getRoot();
			File file = (File)object;
			Asset asset = dataSource.get(file.getAbsolutePath());
			if (asset != null) {
				delegate.apply(object, asset, this, monitor);
			} else {
				asset = delegate.apply(object, this, monitor);
				dataSource.put(file.getAbsolutePath(), asset);
			}
			if (asset != null) {
				apply(object, asset, monitor);
				List result = new ArrayList();
				result.add(asset);
				return result;
			}
		}
		return null;
	}
	
	public void apply(Object object, Asset asset, IProgressMonitor monitor) {
		if (!applyValue(asset)) {
			IRuleDelegate delegate = getDelegate(object);
			if (delegate != null)
				delegate.apply(object, asset, this, monitor);
		}
	}
	
	public boolean applyValue(Asset asset) {
		return false;
	}
	
	public String getFilter() {
		return filter;
	}
	
	public IRuleContainer getParent() {
		return parent;
	}
	
	public IRuleContainer getRoot() {
		if (getParent() instanceof IRule)
			return ((IRule)getParent()).getRoot();
		return getParent();
	}
	
	public String getValue() {
		return value;
	}
	
	public void setFilter(String filter) {
		this.filter = filter;
	}
	
	public void setParent(IRuleContainer parent) {
		this.parent = parent;
	}
	
	public void setValue(String value) {
		if ("".equals(value)) //$NON-NLS-1$
			value = null;
		this.value = value;
	}
	
	public Object[] filter(Object parent) {
		IRuleDelegate delegate = getDelegate(parent);
		if (delegate != null)
			return delegate.filter(parent, filter);
		if (parent instanceof File) {
			File parentFile = (File)parent;
			if (parentFile.isDirectory()) {
				if (filter == null || filter.length() == 0)
					return parentFile.listFiles();
				DirectoryScanner scanner = new DirectoryScanner();
				if (getFilter() != null && getFilter().length() > 0)
					scanner.setIncludes(new String[] {getFilter()});
				scanner.setBasedir(parentFile);
				scanner.scan();
				String[] filePaths = scanner.getIncludedFiles();
				String[] dirPaths = scanner.getIncludedDirectories();
				File[] files = new File[filePaths.length + dirPaths.length];
				for (int i = 0; i <  filePaths.length; i++)
					files[i] = new File(parentFile, filePaths[i]);
				for (int i = 0; i <  dirPaths.length; i++)
					files[i + filePaths.length] = new File(parentFile, dirPaths[i]);
				return files;
			} else {
				
			}
		}
		return null;
	}
	
	public void initialize(Map config) {
		setFilter((String)config.get(FILTER_KEY));
	}

	public void save(Map config) {
		config.put(FILTER_KEY, getFilter());
		config.put(RULE_ID_KEY, getId());
	}
	
	public String toString() {
		StringBuffer buffer = new StringBuffer();
		buffer.append(RuleUtil.getName(getId()));
		if (getValue() != null && getValue().length() > 0) {
			buffer.append(": "); //$NON-NLS-1$
			buffer.append(getValue());
		}
		if (getFilter() != null && getFilter().length() > 0) {
			buffer.append(" ("); //$NON-NLS-1$
			buffer.append(Messages.Batch_FileSystem_Rule_filter);
			buffer.append("=\""); //$NON-NLS-1$
			buffer.append(getFilter());
			buffer.append("\")"); //$NON-NLS-1$
		}
		return buffer.toString();
	}

}
