/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
/*
 *  $RCSfile: $
 *  $Revision: $  $Date: $ 
 */
/**
 * 
 */
package com.ibm.ram.internal.batch.filesystem;

import java.io.*;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.eclipse.core.runtime.IProgressMonitor;

import com.ibm.ram.client.LocalFileArtifact;
import com.ibm.ram.common.data.*;
import com.ibm.ram.internal.client.util.AssetConstants;

/**
 * 
 * @author ebordeau
 * @since 7.1
 */
public class FileRuleDelegate extends AbstractRuleDelegate {

	private static final Logger logger = Logger.getLogger(FileRuleDelegate.class.getName());
	private static final String MANIFEST_FILE_NAME = "manifest.rmd"; //$NON-NLS-1$

	public Asset apply(Object object, IRule rule, IProgressMonitor monitor) {
		return null;
	}

	public void apply(Object object, Asset asset, IRule rule, IProgressMonitor monitor) {
		if (rule instanceof ArtifactRule) {
			File file = (File)object;
			ArtifactRule artifactRule = (ArtifactRule)rule;
			addFileArtifact(file, asset, artifactRule.getArtifactLabel());
		} else if (rule instanceof AttributeRule) {
			addAttributes((File)object, asset, monitor);
		} else if (rule instanceof CategoryRule) {
			addCategories((File)object, asset, monitor);
		} else if (rule instanceof DescriptionRule) {
			setDescription((File)object, asset);
		} else if (rule instanceof OwnerRule) {
			addOwners((File)object, asset, monitor);
		} else if (rule instanceof RelatedAssetRule) {
			addRelatedAssets((File)object, asset, monitor);
		} else if (rule instanceof CommunityRule) {
			setCommunity((File)object, asset, monitor);
		} else if (rule instanceof NameRule) {
			setName((File)object, asset, monitor);
		} else if (rule instanceof ShortDescriptionRule) {
			setShortDescription((File)object, asset, monitor);			
		} else if (rule instanceof AssetTypeRule) {
			setAssetType((File)object, asset, monitor);
		} else if (rule instanceof VersionRule) {
			setVersion((File)object, asset, monitor);
		}
	}
	
	private void setCommunity(File file, Asset asset, IProgressMonitor monitor) {
		String[] lines = getFileContents(file, monitor);
		if (lines.length > 0) {
			CommunityInformation community = new CommunityInformation();
			community.setName(lines[0]);
			asset.setCommunity(community);
		}
	}
	
	private void setName(File file, Asset asset, IProgressMonitor monitor) {
		String[] lines = getFileContents(file, monitor);
		if (lines.length > 0)
			asset.setName(lines[0]);
	}
	
	private void setShortDescription(File file, Asset asset, IProgressMonitor monitor) {
		String[] lines = getFileContents(file, monitor);
		if (lines.length > 0)
			asset.setShortDescription(lines[0]);
	}
	
	private void setAssetType(File file, Asset asset, IProgressMonitor monitor) {
		String[] lines = getFileContents(file, monitor);
		if (lines.length > 0) {
			AssetType assetType = new AssetType();
			assetType.setName(lines[0]);
			asset.setAssetType(assetType);
		}
	}
	
	private void setVersion(File file, Asset asset, IProgressMonitor monitor) {
		String[] lines = getFileContents(file, monitor);
		if (lines.length > 0)
			asset.getIdentification().setVersion(lines[0]);
	}
	
	private String[] getFileContents(File file, IProgressMonitor monitor) {
		BufferedReader reader = null;
		List lines = new ArrayList();
		try {
			reader = new BufferedReader(new FileReader(file));
		} catch (FileNotFoundException e) {
			logger.log(Level.SEVERE, null, e);
		}
		if (reader != null) {
			try {
				String line;
				while ((line = reader.readLine()) != null && !monitor.isCanceled()) {
					lines.add(line);
				}
			} catch (IOException e) {
				logger.log(Level.SEVERE, null, e);
			} finally {
				if (reader != null) {
					try {
						reader.close();
					} catch (IOException e) {}
				}
			}
		}
		return (String[])lines.toArray(new String[lines.size()]);
	}
	
	private void addRelatedAssets(File file, Asset asset, IProgressMonitor monitor) {
		String[] lines = getFileContents(file, monitor);
		for (int i = 0; i < lines.length && !monitor.isCanceled(); i++) {
			String line = lines[i];
			// each line should be in the form of type:guid/version
			int colonIndex = line.indexOf(':');
			if (colonIndex > 0) {
				String type = line.substring(0, colonIndex);
				int slashIndex = line.indexOf('/', colonIndex);
				if (slashIndex > colonIndex) {
					String guid = line.substring(colonIndex + 1, slashIndex);
					String version = line.substring(slashIndex + 1);
					addRelatedAsset(asset, type, guid, version);
				}
			}
		}
	}
	
	private void addRelatedAsset(Asset asset, String relationshipType, String guid, String version) {
		Relationship[] oldRelationships = asset.getRelationships();
		if (oldRelationships == null)
			oldRelationships = new Relationship[0];
		Relationship[] newRelationships = new Relationship[oldRelationships.length + 1];
		System.arraycopy(oldRelationships, 0, newRelationships, 0, oldRelationships.length);
		Relationship relationship = new Relationship();
		relationship.setParentAsset(asset);
		relationship.setRelationshipTypeName(relationshipType);
		relationship.setChildAssetGUID(guid);
		relationship.setChildAssetVersion(version);
		newRelationships[newRelationships.length - 1] = relationship;
		asset.setRelationships(newRelationships);
	}

	private FolderArtifact getArtifactsRoot(Asset asset) {
		FolderArtifact root = asset.getArtifactsRoot();
		if (root == null) {
			root = new FolderArtifact();
			asset.setArtifactsRoot(root);
		}
		return root;
	}
	
	private void addArtifact(Asset asset, Artifact artifact) {
		FolderArtifact root = getArtifactsRoot(asset);
		Artifact[] oldArtifacts = root.getChildren();
		if (oldArtifacts == null)
			oldArtifacts = new Artifact[0];
		Artifact[] newArtifacts = new Artifact[oldArtifacts.length + 1];
		System.arraycopy(oldArtifacts, 0, newArtifacts, 0, oldArtifacts.length);
		newArtifacts[newArtifacts.length - 1] = artifact;
		root.setChildren(newArtifacts);
	}
	
	private void addFileArtifact(File file, Asset asset, String label) {
		if (file.getName().equals(MANIFEST_FILE_NAME))
			return;
		Artifact[] artifacts = getArtifactsRoot(asset).getChildren();
		if (artifacts != null) {
			for (int i = 0; i < artifacts.length; i++) {
				if (artifacts[i].getName().equals(file.getName())) {
					if (label != null && label.length() > 0)
						artifacts[i].setLabel(label);
					return;
				}
			}
		}
		LocalFileArtifact artifact = new LocalFileArtifact(file);
		if (label != null && label.length() > 0)
			artifact.setLabel(label);
		addArtifact(asset, artifact);
	}
	
	public boolean canHandle(Object obj, IRule rule) {
		return obj instanceof File && ((File)obj).isFile();
	}

	private void addAttributes(File file, Asset asset, IProgressMonitor monitor) {
		String[] lines = getFileContents(file, monitor);
		for (int i = 0; i < lines.length && !monitor.isCanceled(); i++) {
			String line = lines[i];
			// each line should be in the form of name=value
			int index = line.indexOf('=');
			if (index > 0) {
				String name = line.substring(0, index);
				String value = line.substring(index + 1);
				addAttribute(asset, name, value);
			}
		}
	}

	private void addAttribute(Asset asset, String name, String value) {
		AssetAttribute attribute = asset.getAssetAttribute(name);
		if (attribute == null) {
			attribute = new AssetAttribute();
			attribute.setName(name);
			AssetAttribute[] oldAttributes = asset.getAssetAttributes();
			if (oldAttributes == null)
				oldAttributes = new AssetAttribute[0];
			AssetAttribute[] newAttributes = new AssetAttribute[oldAttributes.length + 1];
			System.arraycopy(oldAttributes, 0, newAttributes, 0, oldAttributes.length);
			newAttributes[newAttributes.length - 1] = attribute;
			asset.setAssetAttributes(newAttributes);
		}
		String[] values = value.split(","); //$NON-NLS-1$
		attribute.setValues(values);
	}
	
	private void addCategories(File file, Asset asset, IProgressMonitor monitor) {
		String[] lines = getFileContents(file, monitor);
		for (int i = 0; i < lines.length && !monitor.isCanceled(); i++) {
			addSubCategory(asset, lines[i]);
		}
	}
	
	private void addSubCategory(Asset asset, String fullPath) {
		String[] tokens = fullPath.split(AssetConstants.FILE_DELIM);
		if (tokens.length > 2) { // must have schema, category and at least one subcategory
			String schemaName = tokens[0];
			String categoryName = tokens[1];
			String[] subCategories = new String[tokens.length - 2];
			System.arraycopy(tokens, 2, subCategories, 0, subCategories.length);
			addSubCategory(asset, schemaName, categoryName, subCategories);
		}
	}

	private SubCategory addSubCategory(Asset asset, String schemaName, String categoryName, String[] subCategories) {
		Category category = addCategory(asset, schemaName, categoryName);
		SubCategory subCategory = null;
		StringBuffer fullPath = new StringBuffer(schemaName + AssetConstants.FILE_DELIM + categoryName);
		for (int i = 0; i < subCategories.length; i++) {
			fullPath.append(AssetConstants.FILE_DELIM + subCategories[i]);
			SubCategory parent = subCategory;
			SubCategory[] oldSubCategories = null;
			if (parent != null) {
				subCategory = parent.getSubCategory(subCategories[i]);
				oldSubCategories = parent.getSubCategories();
			} else {
				subCategory = category.getSubCategory(subCategories[i]);
				oldSubCategories = category.getSubCategories();
			}
			if (subCategory == null) {
				if (oldSubCategories == null)
					oldSubCategories = new SubCategory[0];
				SubCategory[] newSubCategories = new SubCategory[oldSubCategories.length + 1];
				System.arraycopy(oldSubCategories, 0, newSubCategories, 0, oldSubCategories.length);
				subCategory = new SubCategory();
				subCategory.setName(subCategories[i]);
				subCategory.setFullPath(fullPath.toString());
				newSubCategories[newSubCategories.length - 1] = subCategory;
				if (parent != null)
					parent.setSubCategories(newSubCategories);
				else
					category.setSubCategories(newSubCategories);
			}
		}
		if (subCategory != null)
			subCategory.setSet(true);
		return subCategory;
	}
	
	private Category addCategory(Asset asset, String schemaName, String categoryName) {
		CategorySchema schema = addCategorySchema(asset, schemaName);
		Category category = schema.getCategory(categoryName);
		if (category == null) {
			Category[] oldCategories = schema.getCategories();
			if (oldCategories == null) 
				oldCategories = new Category[0];
			Category[] newCategories = new Category[oldCategories.length + 1];
			System.arraycopy(oldCategories, 0, newCategories, 0, oldCategories.length);
			category = new Category();
			category.setName(categoryName);
			newCategories[newCategories.length - 1] = category;
			schema.setCategories(newCategories);
		}
		return category;
	}
	
	private CategorySchema addCategorySchema(Asset asset, String schemaName) {
		CategorySchema schema = asset.getCategorySchema(schemaName);
		if (schema == null) {
			CategorySchema[] oldSchemas = asset.getCategorySchemas();
			if (oldSchemas == null)
				oldSchemas = new CategorySchema[0];
			CategorySchema[] newSchemas = new CategorySchema[oldSchemas.length + 1];
			System.arraycopy(oldSchemas, 0, newSchemas, 0, oldSchemas.length);
			schema = new CategorySchema();
			schema.setName(schemaName);
			newSchemas[newSchemas.length - 1] = schema;
			asset.setCategorySchemas(newSchemas);
		}
		return schema;
	}
	
	private void setDescription(File file, Asset asset) {
		FileInputStream in = null;
		try {
			in = new FileInputStream(file);
			int length = in.available();
			byte[] bytes = new byte[length];
			in.read(bytes);
			asset.setDescription(new String(bytes));
		} catch (FileNotFoundException e) {
			logger.log(Level.SEVERE, null, e);
		} catch (IOException e) {
			logger.log(Level.SEVERE, null, e);
		} finally {
			if (in != null) {
				try {
					in.close();
				} catch (IOException e) {}
			}
		}
	}
	
	private void addOwners(File file, Asset asset, IProgressMonitor monitor) {
		String[] lines = getFileContents(file, monitor);
		for (int i = 0; i < lines.length && !monitor.isCanceled(); i++) {
			addOwner(asset, lines[i]);
		}
	}
	
	private void addOwner(Asset asset, String uid) {
		UserInformation[] oldOwners = asset.getOwners();
		if (oldOwners == null)
			oldOwners = new UserInformation[0];
		UserInformation[] newOwners = new UserInformation[oldOwners.length + 1];
		System.arraycopy(oldOwners, 0, newOwners, 0, oldOwners.length);
		UserInformation owner = new UserInformation();
		owner.setUid(uid);
		newOwners[newOwners.length - 1] = owner;
		asset.setOwners(newOwners);
	}

}
